package dev.shadowsoffire.apotheosis.ench.table;

package .tmp;

import com.google.common.collect.Lists;
import dev.shadowsoffire.apotheosis.ench.EnchModule;
import dev.shadowsoffire.apotheosis.ench.EnchantmentInfo;
import dev.shadowsoffire.apotheosis.ench.table.ApothEnchantmentMenu;
import dev.shadowsoffire.apotheosis.ench.table.EnchantingStatRegistry;
import dev.shadowsoffire.apotheosis.ench.table.IEnchantableItem;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;
import net.minecraft.Util;
import net.minecraft.util.Mth;
import net.minecraft.util.RandomSource;
import net.minecraft.util.random.WeightedRandom;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.enchantment.Enchantment;
import net.minecraft.world.item.enchantment.EnchantmentHelper;
import net.minecraft.world.item.enchantment.EnchantmentInstance;
import net.minecraftforge.registries.ForgeRegistries;

public class RealEnchantmentHelper {
  public static int getEnchantmentCost(RandomSource rand, int num, float eterna, ItemStack stack) {
    int level = Math.round(eterna * 2.0F);
    if (num == 2)
      return level; 
    float lowBound = 0.6F - 0.4F * (1 - num);
    float highBound = 0.8F - 0.4F * (1 - num);
    return Math.max(1, Math.round(level * Mth.m_216267_(rand, lowBound, highBound)));
  }
  
  public static List<EnchantmentInstance> selectEnchantment(RandomSource rand, ItemStack stack, int level, float quanta, float arcana, float rectification, boolean treasure, Set<Enchantment> blacklist) {
    List<EnchantmentInstance> chosenEnchants = Lists.newArrayList();
    int enchantability = stack.getEnchantmentValue();
    int srcLevel = level;
    if (enchantability > 0) {
      float quantaFactor = 1.0F + getQuantaFactor(rand, quanta, rectification);
      level = Mth.m_14045_(Math.round(level * quantaFactor), 1, (int)(EnchantingStatRegistry.getAbsoluteMaxEterna() * 4.0F));
      ApothEnchantmentMenu.Arcana arcanaVals = ApothEnchantmentMenu.Arcana.getForThreshold(arcana);
      List<EnchantmentInstance> allEnchants = getAvailableEnchantmentResults(level, stack, treasure, blacklist);
      Map<Enchantment, Integer> enchants = EnchantmentHelper.m_44831_(stack);
      allEnchants.removeIf(e -> enchants.containsKey(e.f_44947_));
      List<ArcanaEnchantmentData> possibleEnchants = (List<ArcanaEnchantmentData>)allEnchants.stream().map(d -> new ArcanaEnchantmentData(arcanaVals, d)).collect(Collectors.toList());
      if (!possibleEnchants.isEmpty()) {
        chosenEnchants.add(((ArcanaEnchantmentData)WeightedRandom.m_216822_(rand, possibleEnchants).get()).data);
        removeIncompatible(possibleEnchants, (EnchantmentInstance)Util.m_137509_(chosenEnchants));
        if (arcana >= 25.0F && !possibleEnchants.isEmpty()) {
          chosenEnchants.add(((ArcanaEnchantmentData)WeightedRandom.m_216822_(rand, possibleEnchants).get()).data);
          removeIncompatible(possibleEnchants, (EnchantmentInstance)Util.m_137509_(chosenEnchants));
        } 
        if (arcana >= 75.0F && !possibleEnchants.isEmpty())
          chosenEnchants.add(((ArcanaEnchantmentData)WeightedRandom.m_216822_(rand, possibleEnchants).get()).data); 
        int randomBound = 50;
        if (level > 45)
          level = (int)(srcLevel * 1.15F); 
        while (rand.m_188503_(randomBound) <= level) {
          if (!chosenEnchants.isEmpty())
            removeIncompatible(possibleEnchants, (EnchantmentInstance)Util.m_137509_(chosenEnchants)); 
          if (possibleEnchants.isEmpty())
            break; 
          chosenEnchants.add(((ArcanaEnchantmentData)WeightedRandom.m_216822_(rand, possibleEnchants).get()).data);
          level /= 2;
        } 
      } 
    } 
    return ((IEnchantableItem)stack.m_41720_()).selectEnchantments(chosenEnchants, rand, stack, srcLevel, quanta, arcana, treasure);
  }
  
  public static void removeIncompatible(List<ArcanaEnchantmentData> list, EnchantmentInstance data) {
    Iterator<ArcanaEnchantmentData> iterator = list.iterator();
    while (iterator.hasNext()) {
      if (!data.f_44947_.m_44695_(((ArcanaEnchantmentData)iterator.next()).data.f_44947_))
        iterator.remove(); 
    } 
  }
  
  public static List<EnchantmentInstance> getAvailableEnchantmentResults(int power, ItemStack stack, boolean allowTreasure, Set<Enchantment> blacklist) {
    List<EnchantmentInstance> list = new ArrayList<>();
    IEnchantableItem item = (IEnchantableItem)stack.m_41720_();
    allowTreasure = item.isTreasureAllowed(stack, allowTreasure);
    for (Enchantment enchantment : ForgeRegistries.ENCHANTMENTS) {
      EnchantmentInfo info = EnchModule.getEnchInfo(enchantment);
      if ((!info.isTreasure() || allowTreasure) && info.isDiscoverable() && 
        !blacklist.contains(enchantment) && (
        enchantment.canApplyAtEnchantingTable(stack) || item.forciblyAllowsTableEnchantment(stack, enchantment)))
        for (int level = info.getMaxLevel(); level > enchantment.m_44702_() - 1; level--) {
          if (power >= info.getMinPower(level) && power <= info.getMaxPower(level)) {
            list.add(new EnchantmentInstance(enchantment, level));
            break;
          } 
        }  
    } 
    return list;
  }
  
  public static float getQuantaFactor(RandomSource rand, float quanta, float rectification) {
    float gaussian = (float)rand.m_188583_();
    float factor = Mth.m_14036_(gaussian / 3.0F, -1.0F, 1.0F);
    float rectPercent = rectification / 100.0F;
    if (factor < rectPercent - 1.0F)
      factor = Mth.m_216267_(rand, rectPercent - 1.0F, 1.0F); 
    return quanta * factor / 100.0F;
  }
}
